package org.steveshipway.cheshirecat.item;

import java.util.List;

import net.minecraft.item.Item;

import org.steveshipway.blueprints.Blueprint;
import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.entity.RenderItem;
//import net.minecraft.client.resources.model.ModelResourceLocation;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MathHelper;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;

// Minecraft 1.7
import cpw.mods.fml.common.registry.GameRegistry;
import org.steveshipway.blueprints.util17.*;
// Minecraft 1.8
//import net.minecraftforge.fml.common.registry.GameRegistry;
//import net.minecraft.util.BlockPos;
//import org.steveshipway.blueprints.util18.*;

public class ItemWand extends Item {
	public static String name = "wand";
	boolean enabled = true;
	boolean locked = false;
	
    public ItemWand(){
    	this.setUnlocalizedName(Constants.MODID + "_" + name);
    	// now done with insanely complex resource definitions
    	this.setCreativeTab(CreativeTabs.tabTools);
    	this.setMaxStackSize(1);
    	this.setHarvestLevel("wand",3);
    	// Minecraft 1.7
    	this.setTextureName(Constants.MODID + ":" + name);
    }
    
    public void registerItem() {
    	GameRegistry.registerItem(this, this.name);    	
    }
    public void registerRender() {
// Minecraft 1.8
//    	Minecraft.getMinecraft().getRenderItem().getItemModelMesher().register(
//    			this, 
//    			0, 
//    			new ModelResourceLocation(Constants.MODID + ":" + this.name, "inventory")
//    	);
    }
    
    public String getName() {
    	return name;
    }
    public void enable() { 
    	enabled = true;
    }
    public void disable() { 
    	enabled = false;
    }
    public boolean onItemUse(ItemStack item, EntityPlayer player, World world,
    	// which block was in the target when clicked
// Minecraft 1.7
        int posx, int posy, int posz,
        int sideint,
// Minecraft 1.8
//    	BlockPos pos, 
//    	EnumFacing side, 
    	// where on the target block was clicked (0.0-1.0)
    	float blockx, float blocky, float blockz)
    {
        // Minecraft 1.7
        BlockPos pos = new BlockPos(posx,posy,posz);
        EnumFacing side = Util.enumFromInt(sideint);
    	
        if(world.isRemote) {
    	 	// can't do anything here as we don't own the world (client side)
            return false;
        }
    	NBTTagCompound nbt = item.getTagCompound();
    	if( nbt == null ) {  // no compound data?
    		player.addChatMessage(new ChatComponentText("Wand is not yet enchanted"));
    		return true;
    	}
    	String blueprint = nbt.getString("blueprint");
        if( blueprint == null || blueprint.matches("") ) {
    		// no blueprint is set...
    		player.addChatMessage(new ChatComponentText("Wand is not yet enchanted"));
    		return true;
    	}
   	    // activate the wand!
        // world.spawnEntityInWorld(new EntityLightningBolt(world, x, y, z));
        // we look at what we're pointing at.  The base coord is on top of the block if we're pointing 
        // at the top face, or in front if we're pointing at the side.
        BlockPos target = new BlockPos(pos);
	    int rotation = MathHelper.floor_double((double)(player.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;
	    switch( side ) {
		case DOWN:
        	target = pos.down();
			break;
		case EAST:
        	target = pos.east();
			break;
		case NORTH:
        	target = pos.north();
			break;
		case SOUTH:
        	target = pos.south();
			break;
		case WEST:
        	target = pos.west();
			break;
		case UP:
        	target = pos.up();
			break;
	    }
        if( ! enabled ) {
    		player.addChatMessage(new ChatComponentText("Wait for last build to finish..."));
    		return true;
        }
        enabled = false;        
		//player.addChatMessage(new ChatComponentText("Building on side "+side+" when facing direction "+rotation));
        if( CheshireCat.portfolio.build(world, blueprint, target, rotation) ) {
    		player.addChatMessage(new ChatComponentText("Build was successful"));
        } else {
    		player.addChatMessage(new ChatComponentText("Build failed!"));
        }
        enabled = true;
        return true;
    }
    
    
    public void onCreated(ItemStack itemStack, World world, EntityPlayer player) {
        NBTTagCompound nbt = new NBTTagCompound();
        nbt.setString("blueprint", "");
        nbt.setBoolean("locked",false);
        itemStack.setTagCompound(nbt);
    }
    
    @Override
    public boolean isItemTool(ItemStack itemstack)  {
    	return true;
    }
    
    @Override
    public boolean canItemEditBlocks() {
    	return true;
    }
    
    public void addInformation(ItemStack par1ItemStack, EntityPlayer par2EntityPlayer, List par3List, boolean par4) {
    	NBTTagCompound nbt = par1ItemStack.getTagCompound();
    	if( nbt == null ) {
    		nbt = new NBTTagCompound();
    		nbt.setString("blueprint", "");
            nbt.setBoolean("locked",false);
    		par1ItemStack.setTagCompound(nbt);
    		par3List.add(EnumChatFormatting.RED + "Not initialised");
    		return;
    	}
    	String blueprint = nbt.getString("blueprint");
    	if( blueprint != null && !blueprint.matches("")) {
    		Blueprint bp;
    		bp = CheshireCat.portfolio.blueprint(blueprint);
    		par3List.add(EnumChatFormatting.GREEN + "Creates: " + blueprint);
    		if( bp != null ) {
    			par3List.add(EnumChatFormatting.AQUA + CheshireCat.portfolio.blueprint(blueprint).helpText() );
    		} else {
        		par3List.add(EnumChatFormatting.RED + "INVALID BLUEPRINT" );    		
    		}
    	} else {
    		par3List.add(EnumChatFormatting.AQUA + "Not yet enchanted");    	
    		//par3List.add(EnumChatFormatting.AQUA + "Use /wand set <blueprint>");    	
    	}
    }
    
    public int getItemEnchantability()
    {
        return 50; // it's a wand...
    }

    public boolean getLocked() {
    	return this.locked;
    }
}
