package org.steveshipway.blueprints.directive;

import java.util.HashSet;
import java.util.Random;

import org.apache.logging.log4j.Logger;

import net.minecraft.world.World;





//Minecraft 1.7
//import org.steveshipway.blueprints.util17.*;
//Minecraft 1.8
import org.steveshipway.blueprints.util18.*;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.util.BlockPos;

public class IfBlockDirective extends Directive {
	private IBlockState bs = null;
	private Directive directivea = null;
	private Directive directiveb = null;
	private BlockPos offset = null;
	
	public IfBlockDirective(BlockPos offset, IBlockState bs, Directive da, Directive db, Logger logger) {
		this.directivea = da;
		this.directiveb = db;
		this.bs = bs;
		this.offset = offset;
		if( da == null ) {
    		logger.error("No 'if true' directive given to IF directive");
			return;
		}
		if( db == null ) {
    		logger.error("No 'if false' directive given to IF directive");
			return;
		}
		if( bs == null ) {
    		//logger.warn("No block name given to IF directive");
    		this.bs = Util.getState(Blocks.air,0);
			return;
		}
		if(offset == null) {
			this.offset = new BlockPos(0,0,0);
		}
		
	}
	
	@Override
	public boolean build(World world,BlockPos location, int r, Logger logger, HashSet softlist) {
		IBlockState cbs;
		if( this.directivea == null || this.directiveb == null ) {
    		logger.error("Missing directive in IF directive");
			return false;
		}
		if( this.bs == null ) {
    		logger.error("Invalid comparison block in IF directive");
			return false;
		}
		cbs = Util.getBlockState(world,rotateAndAdd(this.offset,r,location));
		if(cbs.getBlock() == this.bs.getBlock()) {
			logger.info("IfBlock: Match for block type "+Block.blockRegistry.getNameForObject(this.bs.getBlock()));
			return this.directivea.build(world,location,r,logger,softlist);
		} else {
			return this.directiveb.build(world,location,r,logger,softlist);
		}
	}
	
	@Override
	public String asText() {
		String s = "IFBLOCK ";
		int thisDv;
		
		s += Block.blockRegistry.getNameForObject(this.bs.getBlock());
		thisDv = Util.getDvFromState(this.bs);
		if(thisDv>0) {
			s+="("+thisDv+")";
		}
		s += "\n  # if true\n  ";
		
		if( directivea == null) {
			s = s + "NOOP\n  # if false\n  ";
		} else {
			s = s +directivea.asText()+"\n  # if false\n  ";
		}
		if( directiveb == null) {
			s = s + "NOOP";
		} else {
			s = s +directiveb.asText();
		}
		return s;
	}
}
